<!--[if mso]>
	<table cellpadding="0" cellspacing="0" border="0" style="padding: 0; margin: 0; width: 100%;">
		<tr><td colspan="3" style="padding: 0; margin: 0; font-size: 20px; height: 20px;" height="20">&nbsp;</td></tr>
		<tr>
			<td style="padding: 0; margin: 0;">&nbsp;</td>
			<td style="padding: 0; margin: 0;" width="540">
<![endif]-->
{{ content|safe }}
<!--[if mso]>
			</td>
			<td style="padding: 0; margin: 0; font-size: 20px; height: 20px;">&nbsp;</td>
		</tr>
		<tr><td colspan="3" style="padding: 0; margin: 0; font-size: 20px; height: 20px; height: 20px;">&nbsp;</td></tr>
	</table>
<![endif]-->