# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Any

from google.cloud.datastore import Client

from .environ import GOOGLE_DATASTORE_NAMESPACE
from .environ import GOOGLE_SERVICE_PROJECT


class DefaultDatastoreClient(Client):

    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__( # type: ignore
            project=GOOGLE_SERVICE_PROJECT,
            namespace=GOOGLE_DATASTORE_NAMESPACE,
            *args,
            **kwargs
        )