<div style="padding-right: 30px; padding-left: 30px; margin: 32px 0 40px;">
	<table style="border-collapse: collapse; border: 0; background-color: #F4F4F4; height: 70px; table-layout: fixed; word-wrap: break-word; border-radius: 6px;">
		<tr><td style="text-align: center; vertical-align: middle; font-size: 30px;">{{ content|safe }}</td></tr>
	</table>
</div>