# Copyright (C) 2020-2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import fastapi
import pytest

import cbra.core as cbra


def dependency():
    return 1

def f(value: int = fastapi.Depends(dependency)):
    return value


@pytest.mark.asyncio
async def test_run_with_injected_value():
    result = await cbra.run(f)
    assert result == 1